#include "mainWindow.h"
#include "editWindow.h"

mainWindowWidget::mainWindowWidget( QWidget *parent, const char *name )
{
    setCaption("noteZ");

    // check if the directory plain exists, if not
    // create it
    QString d1("Documents/text");
    QDir pd1(d1);
    if (!pd1.exists())
    {
	QDir pd1("Documents");
	if (!pd1.mkdir("text", FALSE))
	{
	    QMessageBox::critical( 0, "noteZ",
				   QString("Can't create directory: \n"+
					   d1 +
					   "\n\nnoteZ will now exit.") );
	    exit (1);
	}
    }
    
    QString d2("Documents/text/plain");
    QDir pd2(d2);
    if (!pd2.exists())
    {
	QDir pd2("Documents/text");
	if (!pd2.mkdir("plain", FALSE))
	{
	    QMessageBox::critical( 0, "noteZ",
				   QString("Can't create directory\n"+
					   d2 +
					   "\n\nnoteZ will now exit.") );
	    exit (1);
	}
    }
    

    unsigned int item, convertCount = 0;
    QString strippedName;
    QDir moveDir("Documents/text");
    moveDir.setFilter(QDir::Files);
    moveDir.setNameFilter("[0-9]*-[0-9]*-[0-9]*-[0-9]*-[0-9]*-[0-9]*.txt");
    
    for (item = 0; item < moveDir.count(); item++) {
	strippedName = moveDir[item].left(moveDir[item].find("."));
	QFile oldFile("Documents/text/" + moveDir[item]);
	QFile newFile("Documents/text/plain/" + strippedName + ".ntz");
	if (!oldFile.open(IO_ReadOnly))
	    QMessageBox::warning(0, "File i/o error", "Could not read\nDocuments/text/" + moveDir[item] + "\nfor conversion.");

	if (!newFile.exists()) {
	    if (!newFile.open(IO_WriteOnly))
		QMessageBox::warning(0, "File i/o error", "Could not write\nDocuments/text/plain/" + strippedName + "\nfor conversion.");

	    QTextStream oldStream(&oldFile);
	    QTextStream newStream(&newFile);

	    while(!oldStream.atEnd())
		newStream << oldStream.read();

	    newFile.close();
	    convertCount++;
	}
	oldFile.close();

    }
    if (convertCount > 0)
	QMessageBox::information(0, "noteZ upgrade", "Converted note files to new format.\nYou may want to remove the\nold files in Documents/text.\n\nSee help for more information.");
    
    QGridLayout *gridLayout = new QGridLayout(this, 2, 3, 10, 10, "gridLayout");
    
    notesList = new QListBox(this, "notesList");
    gridLayout->addMultiCellWidget(notesList, 0, 0, 0, 2);

    QPushButton *newButton = new QPushButton("New", this, "newButton");
    gridLayout->addWidget(newButton, 1, 0);

    QPushButton *editButton = new QPushButton("View/Edit", this, "editButton");
    gridLayout->addWidget(editButton, 1, 1);

    QPushButton *deleteButton = new QPushButton("Delete", this, "deleteButton");
    gridLayout->addWidget(deleteButton, 1, 2);

    this->selected = -1;
    refreshList(notesList);

    QObject::connect(newButton, SIGNAL(clicked()), this, SLOT(newFile()));
    QObject::connect(editButton, SIGNAL(clicked()), this, SLOT(openFile()));
    QObject::connect(deleteButton, SIGNAL(clicked()), this, SLOT(deleteFile()));
    QObject::connect(notesList, SIGNAL(returnPressed(QListBoxItem*)), this, SLOT(openFile()));
    QObject::connect(notesList, SIGNAL(doubleClicked(QListBoxItem*)), this, SLOT(openFile()));
}

void mainWindowWidget::deleteFile()
{

  if( notesList->count() > 0 )
    {
      switch (QMessageBox::warning(0, "Delete file", "Really delete\n'" + notesList->currentText() + "'?",
			           QMessageBox::Yes, QMessageBox::No)) {

      case QMessageBox::Yes:
	this->selected = notesList->currentItem();
	QFile::remove("Documents/text/plain/" + fileList[notesList->currentItem()]);
	refreshList(notesList);
	break;

      case QMessageBox::No:
	// don't delete
	break;
      }
    }
}

void mainWindowWidget::editFile(QString filename, int create)
{

  // editWindowWidget *editWindow = new editWindowWidget(this, "editWindow", true);
  editWindowWidget *editWindow = new editWindowWidget(0, "editWindow", true, 0);

  editWindow->fileName = filename;
  editWindow->loadFile();
  QDialog::DialogCode retval = (QDialog::DialogCode)editWindow->exec();

  if (retval == QDialog::Accepted)
    {
      editWindow->saveFile();
      if( create )
	{
	  // the new selection will be always at the end and count is already
	  // 1 bigger than selected
	  this->selected = notesList->count();
	}
    }

  refreshList(notesList);

}

void mainWindowWidget::newFile()
{

  QDate today(QDate::currentDate());
  QTime now(QTime::currentTime());
  QString name;
  this->editFile(name.sprintf("Documents/text/plain/%4i-%02i-%02i-%02i-%02i-%02i.ntz", today.year(), today.month(), today.day(), now.hour(), now.minute(), now.second()),
		 true );

}

void mainWindowWidget::openFile()
{

  int number = notesList->currentItem();
  
  if( notesList->count() > 0 )
    {
      this->selected = number;
      this->editFile("Documents/text/plain/" + fileList[number], 
		     false);
    }

}

void mainWindowWidget::refreshList(QListBox *liste)
{

  unsigned int item;
  QString zeile;

  liste->clear();

  fileList.setPath("Documents/text/plain/");
  fileList.setFilter(QDir::Files);
  fileList.setSorting(QDir::Name);
  fileList.setNameFilter("*.ntz");

  for (item = 0; item < fileList.count(); item++) {

    QFile datei("Documents/text/plain/" + fileList[item]);

    if (!datei.open(IO_ReadOnly)) {

      QString message = "Could not read item '" + fileList[item] + "'";
      QMessageBox::warning(0, "File i/o error", message);

    } else {

      QTextStream inStream(&datei);

      if (!inStream.atEnd()) {
	zeile = inStream.readLine();
      } else {
	zeile = "<empty file>";
      }

      if (zeile.length() < 1)
	zeile = "<no caption>";

      datei.close();

      liste->insertItem(zeile);

    }

  }
  
  if( notesList->count() > 0 )
    {
      if( this->selected == -1 )
	{
	  notesList->setCurrentItem( 0 );
	}
      else
	{
	  if( notesList->count() > this->selected  )
	    {
	      notesList->setCurrentItem( this->selected );
	    }
	  else
	    {
	      notesList->setCurrentItem( notesList->count()-1 );
	    }
        
	}
    }
  else
    {
      this->selected = -1;
    }

}
